#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE_LENGTH 256
#define NUM_DEVICES 2
#define JSON_FILE_PATH "/opt/fiscalwebserver/appsettings.json"
#define SERVICE_NAME "fws"

// Define the password as a constant
const char* SUDO_PASSWORD = "izgnilkartof";
// Step 1: Define the array of devices with serial numbers and names
char* serial_numbers[NUM_DEVICES] = {"ED315271", "ED322104"};
char* device_names[NUM_DEVICES] = {"ОТ-София", "Едутех"};

// Function to read a specific line from a file
void read_line(const char* filename, int line_number, char* result) {
    FILE* file = fopen(filename, "r");
    if (file == NULL) {
        perror("Error opening file");
        exit(EXIT_FAILURE);
    }

    char line[MAX_LINE_LENGTH];
    int current_line = 0;

    while (fgets(line, sizeof(line), file)) {
        current_line++;
        if (current_line == line_number) {
            strcpy(result, line);
            break;
        }
    }

    fclose(file);
}
// =============================================================================

// Function to stop/start a service
void control_service(const char* service, const char* action) {
    char command[256];
    snprintf(command, sizeof(command), "echo %s | sudo -S service %s %s", SUDO_PASSWORD, service, action);
    system(command);
}
// =============================================================================

// Function to replace the serial number in the JSON file
void replace_serial_in_file(const char* filename, const char* new_serial) {
    FILE* file = fopen(filename, "r+");
    if (file == NULL) {
        perror("Error opening file");
        exit(EXIT_FAILURE);
    }

    // Read the entire file into a buffer
    char buffer[1024];
    fread(buffer, sizeof(char), sizeof(buffer), file);

    // Replace the old serial number with the new one
    char* serial_pos = strstr(buffer, "Serial\": \"");
    if (serial_pos) {
        // Move pointer past "Serial": "
        serial_pos += strlen("Serial\": \""); 
        // Find the ending quote
        char* end_quote = strchr(serial_pos, '\"'); 
        if (end_quote) {
            // Update the serial number
            snprintf(serial_pos, end_quote - serial_pos + 1, "%s", new_serial); 
        }
    }

    // Write the updated buffer back to the file
    fseek(file, 0, SEEK_SET);
    fwrite(buffer, sizeof(char), strlen(buffer), file);
    fclose(file);
}
// =============================================================================

// Function to print ASCII values of each character in a string
void print_ascii_values(const char* str) {
    while (*str) {
        printf("Character: '%c' (ASCII: %d)\n", *str, *str);
        str++;
    }
}
// =============================================================================

int main() {
    // Restart the service
    control_service(SERVICE_NAME, "restart");
    // Declare variable for storing the current serial number from the JSON file
    char current_serial[MAX_LINE_LENGTH] = {0};
    
    while (1) {  // Endless loop
        // Step 2: Read the current serial from the JSON file (line 18)
        read_line(JSON_FILE_PATH, 18, current_serial);
        
        // Log the current serial line read from the JSON
        // Use quotes for better visibility
        printf("Current line read from JSON: '%s'\n", current_serial); 

        // Print ASCII values to check for hidden characters
        print_ascii_values(current_serial);

        // Trim leading whitespace and prepare for extraction
        char *trimmed_serial = current_serial;
        while (*trimmed_serial == ' ') {
            trimmed_serial++;  // Skip leading spaces
        }

        // Extract the serial number from the JSON line
        // Store the extracted serial number
        char extracted_serial[9] = {0}; 
        if (sscanf(trimmed_serial, "\"Serial\": \"%8[^\"]\"", extracted_serial) == 1) {
            // Log the extracted serial number
            printf("Extracted serial number from JSON: %s\n", extracted_serial);
        } else {
            printf("Failed to extract the serial number from the JSON line.\n");
            exit(EXIT_FAILURE); // Exit if the serial number extraction fails
        }

        // Step 3: Create the Zenity dialog
        char zenity_command[1024] = "zenity --list --radiolist --title=\"Избор на активен касов апарат\" --text=\"Касови апарати\" --column \"Избери\" --column \"Устройство\" --width=500 --height=300 ";

        // Populate the Zenity dialog with serial numbers and add the colored dots as symbols
        for (int i = 0; i < NUM_DEVICES; i++) {
            char status[256];  // Store the device description with colored dot
            
            // Log the comparison
            printf("Comparing: %s with %s\n", serial_numbers[i], extracted_serial);
            
            if (strcmp(serial_numbers[i], extracted_serial) == 0) {
                // If matched, add a green dot
                snprintf(status, sizeof(status), "🟢 %s - %s", serial_numbers[i], device_names[i]);
            } else {
                // If not matched, add a red dot
                snprintf(status, sizeof(status), "🔴 %s - %s", serial_numbers[i], device_names[i]);
            }
            
            // Add to the Zenity command
            snprintf(zenity_command + strlen(zenity_command), sizeof(zenity_command) - strlen(zenity_command),
                     "FALSE \"%s\" ", status);
        }

        strcat(zenity_command, " --ok-label=\"OK\" --cancel-label=\"Cancel\"");

        // Step 4: Execute the Zenity command and get the selected serial number
        FILE *zenity_output = popen(zenity_command, "r");
        if (!zenity_output) {
            perror("Error running Zenity");
            exit(EXIT_FAILURE);
        }

        char selected_serial[MAX_LINE_LENGTH] = {0};
        fgets(selected_serial, sizeof(selected_serial), zenity_output);
        pclose(zenity_output);

        // Remove any emojis from the selected_serial (e.g., "🔴 " or "🟢 ")
        char *serial_start = strchr(selected_serial, ' ');
        if (serial_start) {
            serial_start += 1;  // Skip past the space
            strcpy(selected_serial, serial_start);
        }
        
        // Trim newline or extra whitespace
        selected_serial[strcspn(selected_serial, "\n")] = 0;

        // Step 5: Check if the user selected a serial and clicked OK
        if (strlen(selected_serial) > 0) {
            // Find the actual serial number (ignore emoji)
            char selected_device_serial[9] = {0};
            sscanf(selected_serial, "%8s", selected_device_serial);  // Extract serial number from the label

            // Stop the service
            control_service(SERVICE_NAME, "stop");

            // Replace the serial number in the JSON file
            replace_serial_in_file(JSON_FILE_PATH, selected_device_serial);

            // Start the service
            control_service(SERVICE_NAME, "start");

            // Log the update
            printf("Serial updated to: %s\n", selected_device_serial);
            // The loop will continue, so no need to exit or reload the program
        } else {
            printf("No device selected or operation cancelled.\n");
            break; // Exit the loop if Cancel is pressed or OK without selection
        }
    }

    return 0;
}

